/*
 * Windfreak Technologies SynthNV Series RF Signal Generator Library
 * Copyright © 2013 D Scott Guthridge <pdx_scooter@yahoo.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <errno.h>
#include <math.h>
#include <stdarg.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "wfsyn.h"
#include "wfsyn_internal.h"


#define WFSYN_MAX_ERROR	256

/*
 * wfsyn_default_error_function: print errors to stderr
 */
void
wfsyn_default_error_function(const wfsyn_device_type *wfsynp, const char *msg)
/*ARGSUSED*/
{
    (void)fprintf(stderr, "%s\n", msg);
}

/*
 * wfsyn_error: report errors
 */
void
wfsyn_error(const wfsyn_device_type *wfsynp, const char *format, ...)
{
    va_list ap;
    char buffer[WFSYN_MAX_ERROR + 1];

    if (wfsynp->wfsyn_errfn != NULL) {
	va_start(ap, format);
	(void)vsnprintf(buffer, WFSYN_MAX_ERROR, format, ap);
	buffer[WFSYN_MAX_ERROR] = '\000';
	va_end(ap);

	(*wfsynp->wfsyn_errfn)(wfsynp, buffer);
    }
}
