/*
 * Windfreak Technologies SynthNV Series RF Signal Generator Library
 * Copyright © 2013 D Scott Guthridge <pdx_scooter@yahoo.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef WFSYN_INTERNAL_H
#define WFSYN_INTERNAL_H

#ifndef _INTTYPES_H
#include <inttypes.h>
#endif /* _INTTYPES_H */

#ifndef WFSYN_H
#include "wfsyn.h"
#endif /* WFSYN_H */

/*
 * Implementation Limits
 */
#define WFSYN_MAX_LINE	80

/*
 * ADF4350/ADF4351 Register Descriptions
 */
#define ADF435X_R0_INT_MASK		0x7FFF8000
#define ADF435X_R0_INT_SHIFT		15
#define ADF435X_R0_FRAC_MASK		0x00007FF8
#define ADF435X_R0_FRAC_SHIFT		3
#define ADF435X_R0_CONTROL		0x00000000
#define ADF435X_R1_PHASE_ADJUST		0x10000000
#define ADF435X_R1_PRESCALER		0x08000000
#define ADF435X_R1_PHASE_MASK		0x07FF8000
#define ADF435X_R1_PHASE_SHIFT		15
#define ADF435X_R1_MOD_MASK		0x00007FF8
#define ADF435X_R1_MOD_SHIFT		3
#define ADF435X_R1_CONTROL		0x00000001
#define ADF435X_R2_LNLS_MASK		0x60000000
#define ADF435X_R2_LNLS_SHIFT		29
#define ADF435X_R2_MUXOUT_MASK		0x1C000000
#define ADF435X_R2_MUXOUT_SHIFT		26
#define ADF435X_R2_RMUL2		0x02000000
#define ADF435X_R2_RDIV2		0x01000000
#define ADF435X_R2_R_MASK		0x00FFC000
#define ADF435X_R2_R_SHIFT		14
#define ADF435X_R2_DOUBLE_BUFFER	0x00002000
#define ADF435X_R2_ICP_MASK		0x00001E00
#define ADF435X_R2_ICP_SHIFT		9
#define ADF435X_R2_LDF			0x00000100
#define ADF435X_R2_LDP			0x00000080
#define ADF435X_R2_PD_POLARITY		0x00000040
#define ADF435X_R2_POWER_DOWN		0x00000020
#define ADF435X_R2_CP_TRISTATE		0x00000010
#define ADF435X_R2_COUNTER_RESET	0x00000008
#define ADF435X_R2_CONTROL		0x00000002
#define ADF435X_R3_BSCM			0x00800000
#define ADF435X_R3_ABP			0x00400000
#define ADF435X_R3_CHARGE_CANCEL	0x00200000
#define ADF435X_R3_CSR			0x00020000
#define ADF435X_R3_CLKDIV_MODE_MASK	0x00018000
#define ADF435X_R3_CLKDIV_VALUE_MASK	0x00007FF8
#define ADF435X_R3_CLKDIV_VALUE_SHIFT	3
#define ADF435X_R3_CONTROL		0x00000003
#define ADF435X_R4_FEEDBACK_SELECT	0x00800000
#define ADF435X_R4_RF_DIVIDER_MASK	0x00700000
#define ADF435X_R4_RF_DIVIDER_SHIFT	20
#define ADF435X_R4_BSCDV_MASK		0x000FF000
#define ADF435X_R4_BSCDV_SHIFT		12
#define ADF435X_R4_VCO_POWER_DOWN	0x00000800
#define ADF435X_R4_MTLD			0x00000400
#define ADF435X_R4_AUXOUT_SELECT	0x00000200
#define ADF435X_R4_AUXOUT_ENABLE	0x00000100
#define ADF435X_R4_AUXOUT_POWER_MASK	0x000000C0
#define ADF435X_R4_AUXOUT_POWER_SHIFT	6
#define ADF435X_R4_RFOUT_ENABLE		0x00000020
#define ADF435X_R4_RFOUT_POWER_MASK	0x00000018
#define ADF435X_R4_RFOUT_POWER_SHIFT	3
#define ADF435X_R4_CONTROL		0x00000004
#define ADF435X_R5_LDPIN_MODE_MASK	0x00C00000
#define ADF435X_R5_LDPIN_MODE_SHIFT	22
#define ADF435X_R5_RESERVED_ONES	0x00180000
#define ADF435X_R5_CONTROL		0x00000005

#define ADF435X_MUX_3STATE		0x0
#define ADF435X_MUX_DVDD		0x1
#define ADF435X_MUX_DGND		0x2
#define ADF435X_MUX_RDIV		0x3
#define ADF435X_MUX_NDIV		0x4
#define ADF435X_MUX_NALD		0x5
#define ADF435X_MUX_NDLD		0x6

#define ADF435X_BSFMAX			0.125	/* MHz */

#define ADF4350_45FMAX			3000.0	/* MHz */
#define ADF4351_45FMAX			3600.0	/* MHz */
#define ADF4350_PFD_MAX			  32.0	/* MHz */
#define ADF4351_PFD_MAX			  90.0	/* MHz */
#define ADF435X_VCO_MIN			2200.0	/* MHz */
#define ADF435X_VCO_MAX			4400.0	/* MHz */
#define ADF435X_RMUL2_FMAX		  30.0	/* MHz */
#define ADF435X_R_MAX			1023
#define ADF434X_INT_MIN_PRE45		23
#define ADF434X_INT_MIN_PRE89		75
#define ADF435X_INT_MAX			65535
#define ADF435X_MOD_MIN			2
#define ADF435X_MOD_MIN_LS		50
#define ADF435X_MOD_MAX			4095

#define HMC625_ENABLE			0x80
#define HMC625_POWER0			(-13.5)
#define HMC625_POWER63			(+18.0)
#define HMC625_POWER_STEP		0.5
#define HMC625_POWER_MASK		0x3F

/*
 * Spline data indices.
 */
enum {
	X = 0,
	Y = 1
};

/*
 * Spline coefficient indices.
 */
enum {
	B = 0,
	C = 1,
	D = 2
};

/*
 * Capabilities
 */
enum {
    WFSYN_CAP_TEXTCMD			= 0x00000001,
    WFSYN_CAP_ADF4351			= 0x00000002,
    WFSYN_CAP_READMUX			= 0x00000004
};

/*
 * wfsyn_device_type
 */
struct wfsyn_device {
    wfsyn_errfn_type *wfsyn_errfn;		/* error reporting function */
    int wfsyn_fd;				/* serial port descriptor */
    char *wfsyn_device_name;			/* synth device name */
    char *wfsyn_version_string;			/* firmware version string */
    char *wfsyn_model_string;			/* hardware model string */
    char *wfsyn_serial_string;			/* serial number or NULL */
    uint32_t wfsyn_capabilities;		/* capabilities of version */
    double wfsyn_frequency;			/* desired frequency in MHz */
    double wfsyn_power;				/* output power in dBm */
    double wfsyn_power_offset;			/* output power cal. offset */
    wfsyn_options_type wfsyn_current_options;	/* current options */
    wfsyn_options_type wfsyn_new_options;	/* proposed options */
    wfsyn_bool_type wfsyn_pre8_9;		/* 8/9 prescaler flag */
    wfsyn_bool_type wfsyn_rmul2;		/* calculated RMUL2 flag */
    wfsyn_bool_type wfsyn_rdiv2;		/* calculated RDIV2 flag */
    uint16_t wfsyn_r;				/* calculated R value */
    uint16_t wfsyn_mod;				/* calculated MOD value */
    double wfsyn_pfd;				/* computed PFD value */
    uint32_t wfsyn_adf435x[6];			/* current ADF435X registers */
    uint8_t wfsyn_hmc625;			/* current HMC625 value */
    uint8_t wfsyn_refsrc;			/* current refsrc value */
    uint16_t wfsyn_am_step_time;		/* AM step-time in us */
    uint16_t wfsyn_am_reps;			/* AM repetitions */
    uint8_t wfsyn_am_pct;			/* AM "amplitude" */
    uint8_t wfsyn_am_off;			/* AM offset */
    uint16_t wfsyn_pulse_on;			/* pulse on-time */
    uint16_t wfsyn_pulse_off;			/* pulse off-time */
    uint16_t wfsyn_pulse_reps;			/* pulse repetitions */
    char *wfsyn_settings_filename;		/* startup settings file */
    int wfsyn_dcal_segments;			/* #detector spline segments */
    const double (*wfsyn_dcal_xy)[2];		/* detector calibration data */
    const double (*wfsyn_dcal_bcd)[3];		/* detector spline coef. */
};

#ifndef MIN
#define MIN(x, y)   ((x) <= (y) ? (x) : (y))
#endif /* MIN */

#ifndef MAX
#define MAX(x, y)   ((x) >= (y) ? (x) : (y))
#endif /* MAX */

/*
 * WFSYN_POWER_IS_MUTED: test if the given power value is WFSYN_MUTE
 */
#define WFSYN_POWER_IS_MUTED(power)	((power) <= -1000.0)

extern const double adf435x_power_table[];

extern void wfsyn_calc_intmod(const wfsyn_device_type *wfsynp, int *ip_rfdiv,
			      uint16_t *u16p_int, uint16_t *u16p_frac);
extern void wfsyn_calc_power(const wfsyn_device_type *wfsynp,
			     uint8_t *u8_adf435x, uint8_t *u8_hmc625);
extern int wfsyn_calc_rfdiv(const wfsyn_device_type *wfsynp, double frequency);

extern void wfsyn_default_error_function(const wfsyn_device_type *wfsynp,
    					 const char *msg);

extern void wfsyn_error(const wfsyn_device_type *wfsynp,
			const char *format, ...)
#ifdef __GNUC__
			__attribute__ ((format (printf, 2, 3)))
#endif /* __GNUC__ */
;
extern int wfsyn_printf(const wfsyn_device_type *wfsynp,
			const char *format, ...)
#ifdef __GNUC__
			__attribute__ ((format (printf, 2, 3)))
#endif /* __GNUC__ */
;
extern int wfsyn_readline(const wfsyn_device_type *wfsynp, char *buffer,
			  int size);
extern int wfsyn_read_registers(wfsyn_device_type *wfsynp);
extern int wfsyn_read_version(wfsyn_device_type *wfsynp);
extern int wfsyn_spline_calc(int n, const double (*dp)[2], double (*scp)[3]);
extern double wfsyn_spline_eval(int n, const double (*dp)[2],
    				const double (*scp)[3], double x);
extern int wfsyn_update(wfsyn_device_type *wfsynp);
extern int wfsyn_write(const wfsyn_device_type *wfsynp, char *buffer,
		       int length);

#endif /* WFSYN_INTERNAL_H */
