/*
 * Windfreak Technologies SynthNV Series RF Signal Generator Library
 * Copyright © 2013 D Scott Guthridge <pdx_scooter@yahoo.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <errno.h>
#include <math.h>
#include <stdarg.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "wfsyn.h"
#include "wfsyn_internal.h"


/*
 * wfsyn_write: write the given bytes to the device
 *   @wfsynp:  the WFSYN device structure
 *   @buffer: buffer of bytes to write
 *   @length: number of bytes to write
 */
int
wfsyn_write(const wfsyn_device_type *wfsynp, char *buffer, int length)
{
    int rv;

    /*
     * Uncomment to see commands.
     */
    /* (void)printf("sending [%s]\n", buffer); */

    /*
     * Write until all bytes are sent.
     */
    while (length > 0) {
	if ((rv = write(wfsynp->wfsyn_fd, buffer, length)) == -1) {
	    wfsyn_error(wfsynp, "write: %s: %s",
			wfsynp->wfsyn_device_name, strerror(errno));
	    return -1;
	}
	buffer += rv;
	length -= rv;
    }
    return 0;
}

/*
 * wfsyn_printf: print a text command to the device
 *   @wfsynp:  the WFSYN device structure
 *   @format: printf format string
 *   @...: args
 */
int
wfsyn_printf(const wfsyn_device_type *wfsynp, const char *format, ...)
{
    va_list ap;
    char buffer[WFSYN_MAX_LINE + 1];
    int length;

    va_start(ap, format);
    length = vsnprintf(buffer, WFSYN_MAX_LINE, format, ap);
    buffer[WFSYN_MAX_LINE] = '\000';	/* force NUL termination */
    va_end(ap);

    return wfsyn_write(wfsynp, buffer, length);
}
