/*
 * Windfreak Technologies SynthNV Series RF Signal Generator Library
 * Copyright © 2013 D Scott Guthridge <pdx_scooter@yahoo.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <errno.h>
#include <math.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "wfsyn.h"
#include "wfsyn_internal.h"

/*
 * wfsyn_readline: read characters from the device terminated by a newline
 *   @wfsynp:  the WFSYN device structure
 *   @buffer: buffer to hold response
 *   @size:   buffer size
 */
int
wfsyn_readline(const wfsyn_device_type *wfsynp, char *buffer, int size)
{
    int rv;
    char *cp_end = buffer;

    for (;;) {
	int remaining = &buffer[size] - cp_end;

	/*
	 * If the buffer is not full, read.
	 */
	if (remaining == 0) {
	    return -1;
	}
	if ((rv = read(wfsynp->wfsyn_fd, cp_end, remaining)) == -1) {
	    wfsyn_error(wfsynp, "read: %s: %s",
			wfsynp->wfsyn_device_name, strerror(errno));
	    return -1;
	}
	if (rv == 0) {
	    wfsyn_error(wfsynp, "read: %s: unexpected EOF",
			wfsynp->wfsyn_device_name);
	    errno = EIO;
	    return -1;
	}
	cp_end += rv;

	/*
	 * If we found the newline, remove it and break the loop.  If
	 * the newline is preceded by a carriage return, remove that,
	 * too.
	 */
	if (cp_end[-1] == '\n') {
	    *--cp_end = '\000';
	    if (cp_end > buffer && cp_end[-1] == '\r') {
		*--cp_end = '\000';
	    }
	    break;
	}
    }
    return cp_end - buffer;
}
