/*
 * Windfreak Technologies SynthNV Series RF Signal Generator Library
 * Copyright © 2013 D Scott Guthridge <pdx_scooter@yahoo.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <assert.h>
#include <errno.h>
#include <math.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "wfsyn.h"
#include "wfsyn_internal.h"


/*
 * wfsyn_get_frequency: compute the actual output frequency (MHz)
 *   @wfsynp: the WFSYN device structure
 */
double
wfsyn_get_frequency(const wfsyn_device_type *wfsynp)
{
    int shift;
    uint16_t u16_int, u16_frac;

    wfsyn_calc_intmod(wfsynp, &shift, &u16_int, &u16_frac);
    return wfsynp->wfsyn_pfd *
	   ((double)u16_int + (double)u16_frac / (double)wfsynp->wfsyn_mod) /
	   (double)(1U << shift);
}

/*
 * wfsyn_set_frequency: set the desired output frequency (MHz)
 *   @wfsynp: the WFSYN device structure
 *   @frequency: desired frequency in MHz
 */
int
wfsyn_set_frequency(wfsyn_device_type *wfsynp, double frequency)
{
    const wfsyn_options_type *woptp = &wfsynp->wfsyn_current_options;

    /*
     * Make sure the requested frequency is in-range.
     */
    if (frequency < woptp->wopt_fmin || frequency > woptp->wopt_fmax)
    {
	wfsyn_error(wfsynp, "wfsyn_set_frequency: %s: %f MHz: "
			    "frequency out of range",
		    wfsynp->wfsyn_device_name, frequency);
	errno = EDOM;
	return -1;
    }

    /*
     * Update to the new frequency.
     */
    wfsynp->wfsyn_frequency = frequency;
    if (wfsyn_update(wfsynp) == -1)
    	return -1;

    return 0;
}
