/*
 * Windfreak Technologies SynthNV Series RF Signal Generator Library
 * Copyright © 2013 D Scott Guthridge <pdx_scooter@yahoo.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <errno.h>
#include <fcntl.h>
#include <math.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "wfsyn.h"
#include "wfsyn_internal.h"

/*
 * wfsyn_get_power: return the current output power (dBm)
 *   @wfsynp: the WFSYN device structure
 */
double
wfsyn_get_power(const wfsyn_device_type *wfsynp)
{
    uint8_t adf435x;
    uint8_t hmc625;

    wfsyn_calc_power(wfsynp, &adf435x, &hmc625);
    if (adf435x == 0)
	return WFSYN_MUTE;

    adf435x &= ADF435X_R4_RFOUT_POWER_MASK >> ADF435X_R4_RFOUT_POWER_SHIFT;
    return adf435x_power_table[adf435x] + HMC625_POWER0 +
	   (double)(hmc625 & HMC625_POWER_MASK) * HMC625_POWER_STEP -
	   wfsynp->wfsyn_power_offset;
}

/*
 * wfsyn_set_power: set the desired output power (dBm)
 *   @wfsynp: the WFSYN device structure
 *   @power: desired output power in dBm
 */
int
wfsyn_set_power(wfsyn_device_type *wfsynp, double power)
{
    wfsynp->wfsyn_power = power;
    return wfsyn_update(wfsynp);
}

/*
 * wfsyn_get_power_offset: return the output power offset value
 *   @wfsynp: the WFSYN device structure
 */
double
wfsyn_get_power_offset(const wfsyn_device_type *wfsynp)
{
    return wfsynp->wfsyn_power_offset;
}

/*
 * wfsyn_set_power_offset: set the output power offset value
 *   @wfsynp: the WFSYN device structure
 *   @offset: amount added to power in wfsyn_set_power (dBm)
 */
int
wfsyn_set_power_offset(wfsyn_device_type *wfsynp, double offset)
{
	wfsynp->wfsyn_power_offset = offset;
	return 0;
}
