/*
 * Windfreak Technologies SynthNV Series RF Signal Generator Library
 * Copyright © 2013 D Scott Guthridge <pdx_scooter@yahoo.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CALIBRATE_DETECTOR_H
#define CALIBRATE_DETECTOR_H

/*
 * These constants give the default reference level values if none
 * are specified on the command line.
 */
#define DEFAULT_DETECTOR_MIN	-70.0
#define DEFAULT_DETECTOR_MAX	+10.0
#define DEFAULT_DETECTOR_STEP	  5.0

/*
 * MIN_DX: minimum measurement bin width
 */
#define MIN_DX		1.0

/*
 * N_SIGMA: initial bin width is this constant times the standard
 *          deviation from the means over all measurements
 */
#define N_SIGMA		2.0

/*
 * response_type: return value from parse_response
 */
typedef enum {
    RESP_OK	=  0,	/* valid input */
    RESP_SKIP	=  1,	/* skip this value */
    RESP_QUIT	=  2,	/* quit the program */
    RESP_ERROR	=  3 	/* invalid input */
} response_type;

/*
 * detector_measurement_type: a measured power detector value
 */
typedef struct measurement {
    double m_x;		/* mean ADC value */
    double m_y;		/* reference power level */
} measurement_type;


extern response_type set_power_c(double *powerp, const char *cmd);
extern response_type set_power_ci(double *powerp, int *fdp);
extern response_type set_power_i(double *powerp);

extern void measure_power(wfsyn_device_type *wfsynp, double y,
			  measurement_type *mp, double *s2p);

extern void calibrate_detector(wfsyn_device_type *wfsynp, measurement_type *mp,
			       int n_measurements, double width);

#endif /* CALIBRATE_DETECTOR_H */
