/*
 * Windfreak Technologies SynthNV Series RF Signal Generator Library
 * Copyright © 2013 D Scott Guthridge <pdx_scooter@yahoo.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <errno.h>
#include <stdarg.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "error.h"
#include "main.h"
extern int yyget_lineno(void);

/*
 * error: report an error
 */
void
error(error_severity_type severity, const char *format, ...)
{
    va_list ap;
    const char *severity_str;

    switch (severity) {
    case ERR_WARNING:
	severity_str = "warning";
	break;
    case ERR_ERROR:
	severity_str = "error";
	break;
    case ERR_FATAL:
	severity_str = "fatal";
	break;
    default:
	abort();
	/*NOTREACHED*/
    }
    (void)fprintf(stderr, "%s: %s (line %d) %s: ",
    	progname, input_filename, yyget_lineno(), severity_str);
    va_start(ap, format);
    (void)vfprintf(stderr, format, ap);
    va_end(ap);
    (void)fprintf(stderr, "\n");

    if (severity == ERR_FATAL || (severity == ERR_ERROR && !interactive)) {
	exit(3);
	/*NOTREACHED*/
    }
}
