/*
 * Windfreak Technologies SynthNV Series RF Signal Generator Library
 * Copyright © 2013 D Scott Guthridge <pdx_scooter@yahoo.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

%{
#include <assert.h>
#include <ctype.h>
#include "wfsyn.h"
#include "error.h"
#include "y.tab.h"
#include "main.h"

int (*yyinput_fn)(char *buffer, int count);

static void skip_to_eol();

#define YY_INPUT(buffer, result, count) \
	(result) = (*yyinput_fn)((buffer), (count))
%}

%option noyywrap
%option yylineno

%%
[ \t\f\r\v]+					/* whitespace */
"\\\n"						/* line continuation */
"\n"						{ return T_NL; }
[0-9]+						{ return T_INT; }
([0-9]+(\.[0-9]*)|\.[0-9]+)([eE][-+][0-9]+)?	{ return T_FLOAT; }
a1|read-analog1|read_analog1			{ return T_READ_ANALOG1; }
a2|read-analog2|read_analog2			{ return T_READ_ANALOG2; }
am-burst|am_burst				{ return T_AM_BURST; }
--cal|--calibration				{ return T_CALIBRATION; }
-c|--continuous					{ return T_CONTINUOUS; }
-C|--no-continous				{ return T_NO_CONTINUOUS; }
#.*\\\n						/* comment continuation */
#.*						/* comment */
d|detect-power					{ return T_DETECT_POWER; }
d1|digital1					{ return T_DIGITAL1; }
d2|digital2					{ return T_DIGITAL2; }
exit						{ return T_EXIT; }
external					{ return T_EXTERNAL; }
false						{ return T_FALSE; }
f|frequency					{ return T_FREQUENCY; }
fmax						{ return T_FMAX; }
fmin						{ return T_FMIN; }
FRAC-N|FRAC_N					{ return T_FRAC_N; }
fref						{ return T_FREF; }
fres						{ return T_FRES; }
help						{ return T_HELP; }
--hold-time					{ return T_HOLD_TIME; }
icp						{ return T_ICP; }
internal					{ return T_INTERNAL; }
INT-N|INT_N					{ return T_INT_N; }
--log						{ return T_LOG; }
LOW-SPUR|LOW_SPUR				{ return T_LOW_SPUR; }
mode						{ return T_MODE; }
MUTE						{ return T_MUTE; }
pfdmax						{ return T_PFDMAX; }
pfdmin						{ return T_PFDMIN; }
p|power						{ return T_POWER; }
pulse-burst|pulse_burst				{ return T_PULSE_BURST; }
refsrc						{ return T_REFSRC; }
"="						{ return T_ASSIGN; }
"-"						{ return T_MINUS; }
";"						{ return T_SEMI; }
"+"						{ return T_PLUS; }
quit						{ return T_EXIT; }
rmul2en						{ return T_RMUL2EN; }
save-settings|save_settings			{ return T_SAVE_SETTINGS; }
save-to-nvram|save_to_nvram			{ return T_SAVE_TO_NVRAM; }
show						{ return T_SHOW; }
sweep						{ return T_SWEEP; }
true						{ return T_TRUE; }
wlock						{ return T_WLOCK; }
([/[:alnum:]_.][-/[:alnum:]_.]*|\'([^\'\\]|\\.)*\'|\"([^\"\\]|\\.)*\"|\\.)* {
						  return T_STRING; }
.	{
	    if (isprint(yytext[0])) {
		error(ERR_ERROR, "unexpected character '%c'", yytext[0]);
	    } else {
		error(ERR_ERROR, "unexpected character 0x%02x", yytext[0]);
	    }
	    skip_to_eol();
	}
%%

/*
 * skip_to_eol: skip to end of line
 */
void
skip_to_eol()
{
    int c;

    while ((c = input()) != '\n') {
	if (c == EOF)
		break;
    }
    if (c == '\n') {
	unput(c);
    }
}

/*
 * arg_yyin: parse from an argument list
 */
int
arg_yyin(char *buffer, int remaining)
{
    int count = 0;

    assert(remaining > 0);
    for (;;) {
	int length;

	if (parse_argc == 0) {
	    return YY_NULL;
	}
	if (parse_argv[0][parse_offset] == '\000') {
	    *buffer++ = ' ';
	    --remaining;
	    ++count;
	    ++parse_argv;
	    --parse_argc;
	    parse_offset = 0;
	    continue;
	}
	length = strlen(&parse_argv[0][parse_offset]);
	if (remaining > length)
	    remaining = length;
	(void)strncpy(buffer, &parse_argv[0][parse_offset], remaining);
	parse_offset += remaining;;
	count += remaining;
	return count;
    }
}

/*
 * file_yyin: parse from a file
 */
int
file_yyin(char *buffer, int count)
{
    int n = 0;

    while (n < count) {
	int c;

	if ((c = fgetc(yyin)) == EOF)
	    break;

	buffer[n++] = c;
        if (c == '\n' && isatty(fileno(yyin)))
	    break;
    }
    return (n == 0) ? YY_NULL : n;
}
